import falcon
from falcon_rest.conf import settings


# noinspection PyUnusedLocal
def api_error_serializer(req, resp, exception):
    if isinstance(exception, falcon.HTTPError):
        if exception.status in [falcon.HTTP_753, falcon.HTTP_FORBIDDEN]:
            return
    else:  # pragma: no cover
        representation = exception.to_json()
        resp.body = representation
        resp.content_type = 'application/json'


def create(routes, swagger=None, json=True, yaml=True, name='default'):
    middleware = settings.get_middleware()
    for item in middleware:
        item.set_app_name(name)

    app = falcon.API(middleware=middleware)
    for tag_name, tag in routes.items():
        routes_list = tag['routes'] if isinstance(tag, dict) else tag
        for route in routes_list:
            app.add_route('{}{}'.format(settings.API_BASE_PATH, route['path']), route['resource'])

    if swagger and json:
        app.add_route('{}/swagger.json'.format(settings.API_BASE_PATH), swagger.endpoints)

    if swagger and yaml:
        app.add_route('{}/swagger.yaml'.format(settings.API_BASE_PATH), swagger.endpoints)

    app.set_error_serializer(api_error_serializer)
    return app
