import os
import sys
from falcon_rest.contrib.mailer import Mailer


SETTINGS_MODULE_ENVIRONMENT_VARIABLE_NAME = 'FALCON_SETTING_MODULE'


class Settings(object):
    def __init__(self):
        self.__initialized = False

        self.auth_user_model_class = None
        self.middleware = None
        self.auth_module = None
        self.testing_app_creation_func = None
        self.permissions_module = None
        self.auth_user_model_get_func = None

        self.entities = []

        self.settings_module = os.environ.get(SETTINGS_MODULE_ENVIRONMENT_VARIABLE_NAME)

        if not self.settings_module:  # pragma: no cover
            sys.stderr.write('No {} set or is empty'.format(SETTINGS_MODULE_ENVIRONMENT_VARIABLE_NAME))
            exit(-1)

    def __getattr__(self, item):
        if not self.__initialized:
            self.initialize_settings()
            self.__initialized = True
            return getattr(self, item)

        raise AttributeError('There is not such attribute "{}" in settings'.format(item))

    def initialize_settings(self):
        __import__(self.settings_module)
        for k, v in sys.modules[self.settings_module].__dict__.items():
            if k.isupper():
                setattr(self, k, v)

    @staticmethod
    def get_class_attr_from_string_notation(class_string):
        class_module_name, class_name = class_string.rsplit('.', 1)
        __import__(class_module_name)
        cls = sys.modules[class_module_name]
        return getattr(cls, class_name)

    # noinspection PyUnusedLocal
    def get_auth_user_model(self, **kwargs):
        if hasattr(self, 'AUTH_USER_MODEL_GET_FUNC'):
            if self.AUTH_USER_MODEL_GET_FUNC and not self.auth_user_model_get_func:
                self.auth_user_model_get_func = self.get_class_attr_from_string_notation(self.AUTH_USER_MODEL_GET_FUNC)

        if self.auth_user_model_get_func:
            return self.auth_user_model_get_func(**kwargs)

        if not self.auth_user_model_class and self.AUTH_USER_MODEL:
            self.auth_user_model_class = self.get_class_attr_from_string_notation(self.AUTH_USER_MODEL)

        return self.auth_user_model_class

    def get_middleware(self):
        if hasattr(self, 'MIDDLEWARE'):
            if not self.middleware:
                self.middleware = []
                for middleware in self.MIDDLEWARE:
                    self.middleware.append(self.get_class_attr_from_string_notation(middleware)())

        return self.middleware

    def get_auth_module(self):
        if not self.auth_module and self.AUTH_MODULE:
            __import__(self.AUTH_MODULE)
            self.auth_module = sys.modules[self.AUTH_MODULE]
        return self.auth_module

    def get_testing_app_creation_func(self):
        if hasattr(self, 'TESTING_APP_CREATION_FUNC'):
            if not self.testing_app_creation_func:
                self.testing_app_creation_func = self.get_class_attr_from_string_notation(
                    self.TESTING_APP_CREATION_FUNC
                )
        return self.testing_app_creation_func

    def get_permissions_module(self):
        if hasattr(self, 'PERMISSIONS_MODULE'):
            if not self.permissions_module:
                __import__(self.PERMISSIONS_MODULE)
            self.permissions_module = sys.modules[self.PERMISSIONS_MODULE]
        return self.permissions_module

    @staticmethod
    def get_mailer():
        mailing_conf = settings.MAILING

        # if not mailing_conf['port']:
        #     del mailing_conf['port']

        return Mailer(**mailing_conf)

settings = Settings()


def get_default_items_per_page():
    return settings.DEFAULT_ITEMS_PER_PAGE
