from falcon_rest.db import BaseModel, Column, fields, orm
from falcon_rest.schemes import BaseModelScheme as CommonBaseModelScheme, schemes
from . register import FilesRegister
from . models import ModelUsingFiles, FileModelField, FileSchemeField, relationship
from . backends import FileSystemBackEnd, GoogleDriveBackEnd


files_register = FilesRegister()

MODULE_NAME = __name__


def create_file_model_class(tablename, base_model_class=BaseModel, **kwargs):
    tmp_attrs = dict(
        __tablename__='{}__tmp'.format(tablename),
        uuid=Column(fields.String(36), primary_key=True, default=orm.generate_uuid_str()),
        path=Column(fields.String(1024), nullable=False),

        belongs_to_table=Column(fields.String(255), nullable=False),
        belongs_to_field=Column(fields.String(255), nullable=False),

        uploaded_on=Column(fields.DateTime(timezone=True), server_default=orm.func.now()),

        public=Column(fields.Boolean, default=False, server_default='false')
    )

    attrs = dict(
        __tablename__=tablename,

        uuid=Column(fields.String(36), primary_key=True, nullable=False),
        path=Column(fields.String(1024), nullable=False),

        belongs_to_table=Column(fields.String(255), nullable=False),
        belongs_to_field=Column(fields.String(255), nullable=False),
        belongs_to_pk=Column(fields.String(128)),

        uploaded_on=Column(fields.DateTime(timezone=True), nullable=False),
        attached_on=Column(fields.DateTime(timezone=True), server_default=orm.func.now()),

        public=Column(fields.Boolean, default=False, server_default='false')
    )
    tmp_attrs.update(kwargs.get('tmp', kwargs))
    attrs.update(kwargs)

    model_class = type('FileModel', (base_model_class, ), attrs)
    tmp_model_class = type('FileTmpModel', (base_model_class, ), tmp_attrs)

    model_class.tmp_model_class = tmp_model_class
    tmp_model_class.model_class = model_class

    tmp_file_scheme_args = dict(name='FileTmpScheme', meta=dict(exclude=()))
    tmp_file_scheme_args.update(kwargs.get('tmp_file_scheme', {}))
    model_class.tmp_file_scheme = create_scheme(tmp_model_class, **tmp_file_scheme_args)

    tmp_file_upload_scheme_args = dict(
        name='FileTmpUploadScheme', meta=dict(fields=('belongs_to', )),
        belongs_to=schemes.fields.String(
            required=True,
            validate=lambda value: schemes.fields.validate.OneOf(
                files_register.get_belongs_to_values(model_class.tmp_file_scheme),
                error='Not a valid choice {}. Valid choices are: {}'.format(
                    value, ', '.join(files_register.get_belongs_to_values(model_class.tmp_file_scheme))
                )
            )(value)
        )
    )
    tmp_file_upload_scheme_args.update(kwargs.get('tmp_model_class', {}))
    model_class.tmp_file_upload_scheme = create_scheme(tmp_model_class, **tmp_file_upload_scheme_args)

    return model_class


def create_scheme(model_class, name=None, base_model_scheme=CommonBaseModelScheme, **kwargs):
    meta_attrs = dict(
        model=model_class
    )

    if 'meta' in kwargs:
        meta_attrs.update(kwargs['meta'])
        del kwargs['meta']

    attrs = dict(
        Meta=type('Meta', (), meta_attrs)
    )

    attrs.update(kwargs)
    scheme_class = type('{}.{}'.format(MODULE_NAME, name), (base_model_scheme, ), attrs)
    model_class.scheme_class = scheme_class
    return scheme_class
