class AbstractBackEnd(object):
    def __init__(self, get_file_name=None):
        self.get_file_name = get_file_name or self.get_file_name_default

        self.belongs_to_table = None
        self.belongs_to_field = None

    def set_belongs_to(self, belongs_to_table, belongs_to_field):
        self.belongs_to_table = belongs_to_table
        self.belongs_to_field = belongs_to_field
        return self

    # noinspection PyMethodMayBeStatic
    def get_file_name_default(self, file_obj):
        return file_obj.filename

    def write(self, tmp_file):
        """
        Writes file to the backend
        :param tmp_file: Uploaded temporary file object
        :return: path to the file
        """
        raise NotImplementedError('Method "write" should be implemented in any derived class')

    def read(self, file_obj):
        """
        Reads file from the backend
        :param file_obj: Instance of FileModel object
        :return: Tuple (stream, filename, mimetype)
        """
        raise NotImplementedError('Method "read" should be implemented in any derived class')
