from falcon_rest.conf import settings
from falcon_rest.logger import log
from .. abstract_backend import AbstractBackEnd
from . googledrive_api import GoogleDriveApi


class GoogleDriveBackEnd(AbstractBackEnd):
    api = GoogleDriveApi(
        settings.GOOGLE_DRIVE['client_secret_file'], settings.GOOGLE_DRIVE['credential_dir'],
        settings.GOOGLE_DRIVE['application_name'], settings.GOOGLE_DRIVE['service_lifetime'], log
    )

    def __init__(self, folder_id=None, root_folder_id=None):
        super(GoogleDriveBackEnd, self).__init__()
        self.folder_id = folder_id
        self.root_folder_id = (root_folder_id or settings.GOOGLE_DRIVE['root_folder_id'])

    def write(self, tmp_file):
        return self.api.upload(
            tmp_file.stream, tmp_file.filename, tmp_file.headers.get('Content-Type'), self.get_folder_to_upload_to()
        )

    def read(self, file_obj):
        stream = self.api.download(file_obj.path)
        info = self.api.get_file_info(file_obj.path)
        return stream, info['size'], info['name'], info['mimeType']

    def get_folder_to_upload_to(self):
        if self.folder_id:
            return self.folder_id

        if self.root_folder_id and settings.GOOGLE_DRIVE['use_sub_folders']:
            path = self.belongs_to_table, self.belongs_to_field
            folder_id = self.api.get_folder_id_by_path(path, self.root_folder_id)
            if not folder_id:
                folder_id = self.api.create_folders_tree(path, self.root_folder_id)

            return folder_id

        return self.folder_id
