from sqlalchemy import event
from falcon_rest.db import BaseModel
from . models import FileModelField


class FilesRegister(object):
    def __init__(self, base_model=BaseModel):
        self.base_model = base_model
        self.file_models_classes = {}
        self._register = {}

        event.listen(base_model, 'attribute_instrument', lambda cls, key, instr: self.register_file_field(cls, key))

    def register_file_field(self, model_class, field_name):
        column = model_class.get_field(field_name)
        if isinstance(column, FileModelField):
            if model_class.db_label not in self._register:
                self._register[model_class.db_label] = []

            self._register[model_class.db_label].append(
                '{}.{}'.format(model_class.__tablename__, field_name.replace('_file_uuid', ''))
            )

    def get_belongs_to_values(self, scheme_class, table_name=None):
        ret = self._register[scheme_class.Meta.model.db_label]
        if table_name:
            ret = filter(lambda x: x.startswith('{}.'.format(table_name)), ret)
        return ret

    def get_backend(self, belongs_to_table, belongs_to_field):
        for column in self.base_model.metadata.tables[belongs_to_table].columns:
            if column.name == '{}_file_uuid'.format(belongs_to_field):
                return column.backend.set_belongs_to(belongs_to_table, belongs_to_field)
