import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import jinja2


class Mailer(object):
    def __init__(self, host='localhost', port=None, username=None, password=None, use_ssl=False, use_tls=False):
        self.smtp = smtplib.SMTP_SSL(host, port) if use_ssl else smtplib.SMTP(host, port)
        if use_tls:
            self.smtp.starttls()
        self.variables = dict()
        self.subject = None
        self.content_text = None
        self.content_html = None

        if username:
            self.smtp.login(username, password)

    def set_variables(self, variables):
        self.variables = variables
        return self

    def set_content(self, subject, content_text, content_html=None):
        self.subject = jinja2.Template(subject).render(self.variables) if self.variables else subject
        self.content_text = jinja2.Template(content_text).render(self.variables) if self.variables else content_text
        self.content_html = jinja2.Template(content_html).render(self.variables) if self.variables else content_html

        return self

    def send(self, addr_from, addr_to, addr_cc=None):
        if self.content_html:
            msg = MIMEMultipart('alternative')
            msg.attach(MIMEText(self.content_text, 'plain', 'utf-8'))
            msg.attach(MIMEText(self.content_html, 'html', 'utf-8'))
        else:
            msg = MIMEText(self.content_text)

        msg['Subject'] = self.subject
        msg['To'] = addr_to

        msg['From'] = addr_from

        if addr_cc:
            msg['Cc'] = addr_cc

        result = self.smtp.sendmail(addr_from, addr_to, msg.as_string())
        self.smtp.quit()

        return result
