from sqlalchemy.ext.compiler import compiles
from sqlalchemy.types import ARRAY as COMMON_ARRAY
from sqlalchemy.types import TypeDecorator, TEXT, String
from sqlalchemy.dialects.postgresql import ARRAY as POSTGRES_ARRAY


# noinspection PyUnusedLocal
@compiles(COMMON_ARRAY, "sqlite")
def compile_binary_sqlite(type_, compiler, **kw):  # pragma: no cover
    return "TEXT"


# noinspection PyAbstractClass
class ARRAY(TypeDecorator):
    impl = TEXT

    def load_dialect_impl(self, dialect):
        if dialect.name == 'postgresql':  # pragma: no cover
            # TODO: implement support of different types
            return dialect.type_descriptor(POSTGRES_ARRAY(String(200)))
        else:
            return dialect.type_descriptor(TEXT())

    def process_bind_param(self, value, dialect):
        if value is None or dialect.name != 'sqlite':  # pragma: no cover
            return value
        else:
            return ','.join(value)

    def process_result_value(self, value, dialect):
        if value is None or dialect.name != 'sqlite':  # pragma: no cover
            return value
        else:
            return value.split(',')
