from sqlalchemy.ext.compiler import compiles
from sqlalchemy.types import Numeric as CommonNumeric
from sqlalchemy.types import TypeDecorator, String


# noinspection PyUnusedLocal
@compiles(CommonNumeric, "sqlite")
def compile_binary_sqlite(type_, compiler, **kwargs):  # pragma: no cover
    return "VARCHAR(128)"


# noinspection PyAbstractClass
class Numeric(TypeDecorator):
    impl = String

    def load_dialect_impl(self, dialect):
        if dialect.name == 'sqlite':
            return dialect.type_descriptor(String())
        else:
            return dialect.type_descriptor(CommonNumeric())

    def process_bind_param(self, value, dialect):
        if value is None or dialect.name != 'sqlite':
            return value
        else:
            return str(value)

    def process_result_value(self, value, dialect):
        if value is None or dialect.name != 'sqlite':
            return value
        else:
            return float(value)
