from falcon_rest.conf import settings


def get_entity_end_points(
        module, entity, id_field, put_under='',
        has_create=True, has_info=True, has_modify=True, has_delete=True, has_list=True, list_suffix='/list',
        permissions=None
):
    if settings.COLLECT_ENTITIES:
        permissions = permissions or []

    if put_under:
        put_under = '/{}'.format(put_under)

    end_points = []

    if has_create:
        end_points.append(
            {'path': '{}/{}/create'.format(put_under, entity), 'resource': module.Create(), 'method': 'post'}
        )
        if settings.COLLECT_ENTITIES:
            permissions.append('create')

    if hasattr(module, 'Create'):
        module.Create.has_create = has_create

    methods = []
    if has_info:
        methods.append('get')
        if settings.COLLECT_ENTITIES:
            permissions.append('show')

    if has_modify:
        methods.append('patch')
        if settings.COLLECT_ENTITIES:
            permissions.append('patch')

    if has_delete:
        methods.append('delete')
        if settings.COLLECT_ENTITIES:
            permissions.append('delete')

    if has_info or has_modify or has_delete:
        end_points.append({
            'path': '{}/{}/{{{}}}'.format(put_under, entity, id_field)
                    if isinstance(id_field, str) else '{}/{}/'.format(put_under, entity) + '/'.join(
                        ['{{{}}}'.format(fld) for fld in id_field]
                    ),
            'resource': module.Resource(has_info, has_modify, has_delete),
            'methods': methods
        })

    if has_list:
        end_points.append({
            'path': '{}/{}{}'.format(put_under, entity, list_suffix), 'resource': module.List(), 'method': 'get'
        })
        if settings.COLLECT_ENTITIES:
            permissions.append('list')

    if settings.COLLECT_ENTITIES:
        add_entity(
            module, id_field, '{}/{}'.format(put_under, entity) if put_under else '/{}'.format(entity), permissions
        )

    return end_points

ge2p = get_entity_end_points


def ge2p_no_write(mod, entity, id_field, put_under=''):
    return ge2p(mod, entity, id_field, put_under, has_create=False, has_modify=False, has_delete=False)


def ge2p_no_read(mod, entity, id_field, put_under=''):
    return ge2p(mod, entity, id_field, put_under, has_info=False)


def ge2p_list_only(mod, entity, id_field, put_under='', **kwargs):
    return ge2p(
        mod, entity, id_field, put_under, has_create=False, has_info=False, has_modify=False, has_delete=False,
        list_suffix='', **kwargs
    )


def check_permission(endpoint_or_entity_name, req, permission, obj=None):
    if obj and hasattr(obj, 'can'):
        return obj.can(req, permission)

    permissions_module = settings.get_permissions_module()
    if not permissions_module:  # pragma: no cover
        return True
    permission_func = 'has_{}_permission'.format(permission)

    if not hasattr(permissions_module, permission_func):  # pragma: no cover
        return True

    if isinstance(endpoint_or_entity_name, str):
        entity_name = endpoint_or_entity_name
    else:
        entity_name = endpoint_or_entity_name.scheme_class.__name__.replace(
            'SchemeModify', ''
        ).replace('SchemeGet', '').replace('Scheme', '')

    if permission in ['create', 'list']:
        return getattr(permissions_module, permission_func)(entity_name, req)
    else:
        return getattr(permissions_module, permission_func)(entity_name, req, obj)


def add_entity(module, id_field, path, permissions):
    entity = None
    for attr_name, attr in module.__dict__.items():
        if attr_name.endswith('Model') and attr_name != 'BaseModel':
            if attr_name.lower() == '{}model'.format(module.__name__.rsplit('.', 1)[1].replace('_', '')):
                entity = attr
                break

    settings.entities.append(dict(
        model=entity,
        id_field=id_field,
        name=entity.get_name(),
        path=path,
        permissions=permissions
    ))


def get_request_ip(req):
    try:
        return req.access_route[0]
    except IndexError:  # pragma: no cover
        return None
