import logging
from falcon_rest.conf import settings


class Log(object):
    __loggers = None

    def __getattr__(self, item):
        if not self.__loggers:
            self.initialize_logging()
            self.__initialized = True

        return getattr(self.__loggers['default'], item)

    def initialize_logging(self):
        if hasattr(settings, 'LOGGING'):
            for logger_name, logger in settings.LOGGING.items():
                self.initialize_logger(
                    logger_name, level=logger.get('level', 'error'), filename=logger.get('filename'),
                    log_to_console=logger.get('log_to_console'), fmt=logger.get('format')
                )
        else:
            self.initialize_logger(
                level=getattr(settings, 'LOG_LEVEL'), filename=getattr(settings, 'LOG_FILE'),
                log_to_console=getattr(settings, 'LOG_TO_CONSOLE'), fmt=getattr(settings, 'LOG_FORMAT')
            )

    def initialize_logger(self, name='default', level='error', filename=None, log_to_console=None, fmt=None):
        if not self.__loggers:
            self.__loggers = {}

        self.__loggers[name] = logging.getLogger(name)
        formatter = logging.Formatter(fmt)

        if filename:
            file_handler = logging.FileHandler(filename=filename)
            file_handler.setFormatter(formatter)
            self.__loggers[name].addHandler(file_handler)

        if log_to_console:
            stream_handler = logging.StreamHandler()
            stream_handler.setFormatter(formatter)
            self.__loggers[name].addHandler(stream_handler)

        self.__loggers[name].setLevel(getattr(logging, level.upper()))


log = Log()
