import re


FROM_CAMEL_CASE_TO_UNDERSCORE_RE = re.compile(r'\B([A-Z])')


class Error(object):
    def __init__(self, code, message='', reason=''):
        self.code = code
        self.message = message
        self.reason = reason

    def set_reason(self, reason):
        self.reason = FROM_CAMEL_CASE_TO_UNDERSCORE_RE.sub('_\\1', reason).lower()
        return self

    def set_message(self, message):
        self.message = message
        return self


class CommonErrors(object):
    __start_from = 1000

    MalformedJson = Error(10, 'Malformed JSON')
    NoSuchAttribute = Error(20, 'No such attribute')
    PermissionError = Error(30, 'Can\'t create file')
    DataError = Error(40, 'data_error')

    @classmethod
    def get_already_exists_error(cls, entity, unique_field):
        return type(
            'AlreadyExists', (Error, ),
            dict(
                code=1000, reason='already_exists',
                message='{} with such {} already exists'.format(entity, unique_field)
            )
        )

    @classmethod
    def get_fk_violation_error(cls, entity, pk, column_name, pk_value):
        return type(
            'FkViolation', (Error, ),
            dict(
                code=1100, reason='fk_violation',
                message='{} with {} = {} passed in {}, does not exist in the Database'.format(
                    ''.join(e.title() for e in entity.split('_')), pk, pk_value, column_name
                )
            )
        )

    @classmethod
    def get_pre_condition_failed_error(cls, error_message):
        return type(
            'PreConditionFailed', (Error, ), dict(code=1200, reason='pre_condition_failed', message=error_message)
        )


class AuthErrors(object):
    __start_from = 100

    IncorrectCredentials = Error(100, 'Can\'t find user with credentials provided')
    AuthTokenNotProvided = Error(110, 'No header X-Auth-Token present')
    IncorrectTokenProvided = Error(120, 'Incorrect token provided')
    TokenExpired = Error(130, 'Token expired')
    Forbidden = Error(140, 'You don\'t have permissions to access this resource')


class UserErrors(object):
    __start_from = 200

    IncorrectPassword = Error(200, 'Incorrect password')
    CantRefreshToken = Error(210, 'Can\'t check and refresh token')


def set_reason_for_error_from_errors_collection_attribute_name():
    classes = globals()

    for cls in classes:
        if 'Errors' not in cls:
            continue

        for attr in dir(classes[cls]):
            if attr.startswith('_') or not hasattr(getattr(classes[cls], attr), 'set_reason'):
                continue

            getattr(classes[cls], attr).set_reason(attr)


set_reason_for_error_from_errors_collection_attribute_name()
