from marshmallow_sqlalchemy import ModelSchema
from falcon_rest.db import get_db
from falcon_rest.responses import ObjectCreatedResponse
from . schemes import ObjectCreatedUuidAsPk, ObjectCreatedCompositeOrStrPk


class BaseModelScheme(ModelSchema):
    def get_db(self):
        return get_db(self.Meta.model.db_label if hasattr(self.Meta.model, 'db_label') else 'default')

    def load(self, data, session=None, instance=None, *args, **kwargs):
        return super(BaseModelScheme, self).load(data, self.get_db().session, instance, *args, **kwargs)

    def validate(self, data, session=None, *args, **kwargs):
        return super(BaseModelScheme, self).validate(data, self.get_db().session, *args, **kwargs)

    @classmethod
    def get_object_created_response(cls, **kwargs):
        if 'scheme_class' in kwargs:
            scheme_class = kwargs['scheme_class']
            del kwargs['scheme_class']
        else:
            scheme_class = cls

        pk_field_name = scheme_class.Meta.model.get_model_class_primary_key()
        pk_field = scheme_class.Meta.model.get_field(pk_field_name)
        pk_type = str(pk_field.type).lower()

        if 'int' in pk_type:
            pass
        elif 'varchar' in pk_type:
            if 'uuid' in pk_field_name:
                if 'scheme' not in kwargs:
                    kwargs['scheme'] = ObjectCreatedUuidAsPk
                if 'data_key' not in kwargs:
                    kwargs['data_key'] = 'object_uuid'
            else:
                if 'scheme' not in kwargs:
                    kwargs['scheme'] = ObjectCreatedCompositeOrStrPk
                if 'data_key' not in kwargs:
                    kwargs['data_key'] = 'object_pk'

        return ObjectCreatedResponse(**kwargs)
