from marshmallow import Schema, fields


CHOICES_VALIDATION_ERR_MESSAGE = 'Not a valid value ({input}), valid values are [{choices}] for [{labels}] accordingly'


class SuccessScheme(Schema):
    success = fields.Constant(True, type='boolean')


class ObjectCreated(SuccessScheme):
    object_id = fields.Integer()


class ObjectCreatedUuidAsPk(SuccessScheme):
    object_uuid = fields.String()


class ObjectCreatedCompositeOrStrPk(SuccessScheme):
    object_pk = fields.String()


class ObjectScheme(SuccessScheme):
    payload = fields.Dict()


class ObjectsListSchemeInner(Schema):
    total = fields.Integer()
    page = fields.Integer()
    per_page = fields.Integer()
    items = fields.Dict(many=True)


class ObjectsListScheme(SuccessScheme):
    payload = fields.Nested(ObjectsListSchemeInner())


class ErrorResponse(Schema):
    def dump(self, obj, many=None, update_fields=True, **kwargs):
        result = super(ErrorResponse, self).dump(obj, many, update_fields, **kwargs)
        result.data['success'] = False
        return result


class ValidationError(ErrorResponse):
    success = fields.Constant('false', type='boolean')
    error_type = fields.Constant('validation_error')
    errors = fields.List(
        fields.Dict, description='Key is the field name and value is the list of the validation errors for this fields'
    )


class OperationErrorInner(Schema):
    code = fields.Integer()
    reason = fields.String()
    message = fields.String()


class OperationError(ErrorResponse):
    success = fields.Constant('false', type='boolean')
    error_type = fields.Constant('operation_error')
    error = fields.Nested(OperationErrorInner())


class AttributeNotExists(ErrorResponse):
    success = fields.Constant('false', type='boolean')
    error_type = fields.Constant('attribute_not_exists')
    attribute = fields.String()


class ObjectNotFoundError(ErrorResponse):
    success = fields.Constant('false', type='boolean')
    error_type = fields.Constant('not_found_error')


class ForbiddenError(OperationError):
    error_type = fields.Constant('forbidden_error')


class UnauthenticatedError(OperationError):
    error_type = fields.Constant('unauthenticated_error')


class TokenExpiredError(OperationError):
    error_type = fields.Constant('token_expired_error')


class MalformedJsonError(OperationError):
    error_type = fields.Constant('malformed_json_error')
