from apispec.core import APISpec


class APISpecWithSecurityDefinitions(APISpec):
    def __init__(self, title, version, plugins=(), info=None, **options):
        super(APISpecWithSecurityDefinitions, self).__init__(title, version, plugins, info, **options)
        self._security_definitions = {}

    def security_definition(self, name, properties=None):
        """Add a new security_definition to the spec.

        .. note::

            If you are using `apispec.ext.marshmallow`, you can pass fields' metadata as
            additional keyword arguments.

            For example, to add ``enum`` to your field: ::

                status = fields.String(required=True, enum=['open', 'closed'])

        https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#security-definitions-object
        """
        self._security_definitions[name] = properties

    def to_dict(self):  # pragma: no cover
        ret = super(APISpecWithSecurityDefinitions, self).to_dict()
        ret['securityDefinitions'] = self._security_definitions
        return ret
