from falcon_rest.testing import get_random_object
from . api_base_test_case import ApiBaseTestCase
from api_dnl import settings
from api_dnl.entities.user.user import UserScheme
from api_dnl.helpers import create_admin_user
from api_dnl.tests import FAST_TESTING


class ApiAuthorizedTestCase(ApiBaseTestCase):
    token = None
    mail_or_name = None
    password = None
    need_expired_token = False

    # noinspection PyAttributeOutsideInit
    def setUp(self):
        super(ApiAuthorizedTestCase, self).setUp()
        if settings.FAST_TESTING:
            user = get_random_object(
                UserScheme, login_ip='127.0.0.1', password=FAST_TESTING['hashed_password'], name='admin'
            )
            user.save()
            self.email_or_name, self.password = user.email, FAST_TESTING['plain_password']
            self.token = FAST_TESTING['expired_token' if self.need_expired_token else 'token']
        else:
            self.email_or_name, self.password = create_admin_user('admin')
            self.set_token(self.email_or_name, self.password)

    def set_token(self, email_or_name, password):
        result = super(ApiAuthorizedTestCase, self).post(
            settings.AUTH_END_POINT, {'email_or_name': email_or_name, 'password': password}
        )
        self.token = result.json['payload']['token']

    def add_token_to_header(self, headers=None):
        if headers is None:
            headers = {}

        headers.update({'x-auth-token': self.token})
        return headers

    def get(self, path, headers=None, params=None):
        return super(ApiAuthorizedTestCase, self).get(path, self.add_token_to_header(headers), params=params)

    def post(self, path, data=None, headers=None):
        return super(ApiAuthorizedTestCase, self).post(path, data, self.add_token_to_header(headers))

    def patch(self, path, data=None, headers=None):
        return super(ApiAuthorizedTestCase, self).patch(path, data, self.add_token_to_header(headers))

    def delete(self, path, headers=None):
        return super(ApiAuthorizedTestCase, self).delete(path, self.add_token_to_header(headers))
