import simplejson as json
import falcon
from falcon_rest import schemes
from falcon_rest.conf import settings
from .temporary_db_test_case import TemporaryDBTestCase


class ApiBaseTestCase(TemporaryDBTestCase):
    APP = None

    # noinspection PyAttributeOutsideInit
    def setUp(self):
        super(ApiBaseTestCase, self).setUp()
        if not ApiBaseTestCase.APP:
            ApiBaseTestCase.APP = settings.get_testing_app_creation_func()()

        self.app = ApiBaseTestCase.APP

    @staticmethod
    def get_assert_success___status(result):
        return result.status, falcon.HTTP_200

    def get_assert_object_created___body(self, result):
        return self.get_assert___body(result, schemes.ObjectCreated)

    def get_assert_object_info___body(self, result):
        return self.get_assert___body(result, schemes.ObjectCreated)

    @staticmethod
    def get_assert_validation_error___status(result):
        return result.status, falcon.HTTP_400

    def get_assert_validation_error___body(self, result):
        return self.get_assert___body(result, schemes.ValidationError)

    @staticmethod
    def get_assert_operation_error___status(result):
        return result.status, falcon.HTTP_406

    def get_assert_operation_error___body(self, result):
        return self.get_assert___body(result, schemes.OperationError)

    @staticmethod
    def get_assert_not_found_error___status(result):
        return result.status, falcon.HTTP_404

    @staticmethod
    def get_assert_forbidden_error___status(result):
        return result.status, falcon.HTTP_FORBIDDEN

    def get_assert_not_found_error___body(self, result):
        return self.get_assert___body(result, schemes.ObjectNotFoundError)

    def get_assert_just_ok___body(self, result):
        return self.get_assert___body(result, schemes.SuccessScheme)

    @staticmethod
    def get_assert___body(result, scheme):
        return scheme().dump(result.json).data, result.json

    @staticmethod
    def get_assert_list___body(result):
        return schemes.ObjectsListScheme().dump(result.json).data, result.json

    def get(self, path, headers=None, params=None):
        return self.request('get', self.get_path(path), headers=headers, params=params)

    def post(self, path, data=None, headers=None):
        return self.request('post', self.get_path(path), json.dumps(data) if data is not None else None, headers)

    def patch(self, path, data=None, headers=None):
        return self.request('patch', self.get_path(path), json.dumps(data) if data is not None else None, headers)

    def delete(self, path, headers=None):
        return self.request('delete', self.get_path(path), headers=headers)

    def request(self, method, path, data=None, headers=None, params=None):
        if method == 'get':
            return self.simulate_get(path, headers=(headers or dict()), params=(params or dict()))
        elif method == 'post':
            return self.simulate_post(path, body=data, headers=headers if headers is not None else dict())
        elif method == 'patch':
            return self.simulate_patch(path, body=data, headers=headers if headers is not None else dict())
        elif method == 'delete':
            return self.simulate_delete(path, headers=headers if headers is not None else dict())

    @staticmethod
    def get_path(path):
        return path if path.startswith(settings.API_BASE_PATH) else '{}{}'.format(settings.API_BASE_PATH, path)