from . create_object import BaseCreateObject


class BaseResourceModifyTestCase(BaseCreateObject):
    has_update_by = True
    has_unique_field = True

    endpoint = None
    resource_endpoint = None
    resource_endpoint_not_found = None

    __test__ = False

    # noinspection PyPep8Naming
    @classmethod
    def setUpClass(cls):
        if not cls.resource_endpoint:
            cls.resource_endpoint = '{}/1'.format(cls.endpoint)

        if not cls.resource_endpoint_not_found:
            cls.resource_endpoint_not_found = '{}/not_existent_id'.format(cls.endpoint)

    def modify_object(self, obj):
        raise NotImplementedError('Method should be implemented in any derived class')

    def test_resource__not_found_on_update_error(self):
        result = self.patch(self.resource_endpoint_not_found, {})
        self.assertEqual(*self.get_assert_not_found_error___status(result))
        self.assertDictEqual(*self.get_assert_not_found_error___body(result))

    def test_resource__modify(self):
        obj = self.create_object()
        if self.has_update_by:
            self.assertIsNone(getattr(obj, self.get_update_by_field()))

        result = self.modify_object(obj)
        if isinstance(result, dict):
            for k, v in result.items():
                setattr(obj, k, v)

        self.patch(self.resource_endpoint, self.scheme_class().dump(obj).data)

        if self.has_update_by:
            obj = self.model_class.get_object_by_id(1)
            self.assertEqual(getattr(obj, self.get_update_by_field()), 'admin')

    def test_resource__delete(self):
        self.create_object()
        result = self.delete(self.resource_endpoint)
        self.assertEqual(*self.get_assert_success___status(result))
        self.assertDictEqual(*self.get_assert_just_ok___body(result))

        result = self.delete(self.resource_endpoint)
        self.assertEqual(*self.get_assert_not_found_error___status(result))
        self.assertDictEqual(*self.get_assert_not_found_error___body(result))

    @staticmethod
    def get_update_by_field():
        return 'update_by'
